// Serializable.h: interface for the Serializable class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SERIALIZABLE_H__3D10D1A7_F0E2_11D3_BB4A_00A0C941C02C__INCLUDED_)
#define AFX_SERIALIZABLE_H__3D10D1A7_F0E2_11D3_BB4A_00A0C941C02C__INCLUDED_

#define VISUAL_CPP
#ifdef VISUAL_CPP // Implementation dependent declerations
// Ignore warning about STL long symbols, ok to ignore.
#pragma warning( disable : 4786 )
#endif // VISUAL_CPP

#include <fstream.h>
#include <vector>
#include "IOException.h"
#include "SerializationException.h"

/** An interface for stream serialization support.
*/
class Serializable
{
public:
	Serializable() throw() {}
	virtual ~Serializable() throw() {}
	
	// NOTE: After a call to Read, the validity of the input 
	// that was read from the stream should be checked!

	/// Reads the object from the input stream.
	virtual void Read(ifstream & stream)
		throw(IOException, SerializationException) = 0;
	/// Writes the object to the output stream.
	virtual void Write(ofstream & stream) const
		throw(IOException) = 0;
	
	/// serializes an object to a stream
	friend ofstream & operator << (ofstream & stream, const Serializable & object)
		throw(IOException)
	{
		if (stream.is_open() == false)
		{
			throw IOException("operator << : stream is not open!");
		}
		object.Write(stream);
		return stream;
	}
	
	/// serializes an object from a stream
	friend ifstream & operator >> (ifstream & stream, Serializable & object)
		throw(IOException)
	{
		if (stream.is_open() == false)
		{
			throw IOException("operator >> : stream is not open!");
		}
		object.Read(stream);
		return stream;
	}
};

/// serializes a vector of serializable objects
template<class T>
inline void WriteVector(ofstream & stream, const std::vector<T> & objectVector)
{
	for (std::vector<T>::const_iterator objectIter = objectVector.begin();
	objectIter != objectVector.end();
	++objectIter)
	{
		stream << (*objectIter) << " ";
	}
	stream << endl;
}

/** An implementation of the insertion operator for standard strings. 
* return the specified stream.
* param stream the input stream.
* param str the string that stores the infromation from the stream.
*/
ifstream & operator>>(ifstream & stream, std::string & str);

/** An implementation of the extraction operator for standard strings. 
* return the specified stream.
* param stream the output stream.
* param str the string that is to be written to the stream.
*/
ofstream & operator<<(ofstream & stream, std::string & str);


#endif // !defined(AFX_SERIALIZABLE_H__3D10D1A7_F0E2_11D3_BB4A_00A0C941C02C__INCLUDED_)
