#include "x3dExporter.h"
#include "element.h"
#include "attribute.h"

#ifdef _DEBUG
int Element::m_count = 0;
#endif

void Element::init()
{
#ifdef _DEBUG
    m_count++;
#endif
}

void Element::addAttribute(const Attribute * attribute, bool back)
{
    if (back) m_attributes.push_back(attribute);
    else  m_attributes.push_front(attribute);
}

void Element::addElement(const Element * element, bool back)
{
    if (back) m_elements.push_back(element);
    else m_elements.push_front(element);
}

void Element::merge(const Element * element)
{
    for (AttributeList::const_iterator  attrIter = element->m_attributes.begin();
         attrIter != element->m_attributes.end(); attrIter++)
    {
        addAttribute((*attrIter));
    }

    for (ElementList::const_iterator elemIter = element->m_elements.begin();
         elemIter != element->m_elements.end(); elemIter++)
    {
        addElement(*elemIter);
    }
}

void Element::exportXml() const
{
    if (m_elements.empty()) {
        if (m_attributes.empty()) {
            exEmptyElement(m_tag);
            return;
        }

        exOpenElement(m_tag);
        // export attributes:
        for (AttributeList::const_iterator  attrIter = m_attributes.begin();
             attrIter != m_attributes.end(); attrIter++)
        {
            (*attrIter)->exportXml();
        }
        exCloseEndElement();
        return;
    }

    if (m_attributes.empty()) {
        exStartElement(m_tag);
        // export elements:
        for (ElementList::const_iterator elemIter = m_elements.begin();
             elemIter != m_elements.end(); elemIter++)
        {
            (*elemIter)->exportXml();
        }
        exEndElement(m_tag);
        return;
    }

    exOpenElement(m_tag);
    // export attributes:
    for (AttributeList::const_iterator attrIter = m_attributes.begin();
         attrIter != m_attributes.end(); attrIter++) {
        (*attrIter)->exportXml();
    }
    exCloseElement();
    // export elements:
    for (ElementList::const_iterator  elemIter = m_elements.begin();
         elemIter != m_elements.end(); elemIter++) {
        (*elemIter)->exportXml();
    }
    exEndElement(m_tag);
}
