#include "myString.h"

int myString::length(const char * str, bool literal)
{
    int newSize = 0;
    char oldChar = 0;
    for (char curChar = *str; curChar != '\0'; curChar = *str++) {
        switch (curChar) {
          case '\\': break;
          case '&': newSize += 5; break;
          case '<': newSize += 4; break;
          case '>': newSize += 4; break;
          case '\"': 
          case '\'': if (oldChar == '\\') newSize += 6; break;
          case ' ': m_hasSpace = true;
          default: newSize++; break;
        }
        oldChar = curChar;
    }
    if (m_hasSpace) newSize += 2;
    return newSize;
}

void myString::copy(const char * str, bool literal, int start)
{
    if (m_hasSpace) m_str[start++] = '\"';
    char oldChar = 0;
    for (char curChar = *str; curChar != '\0'; curChar = *str++) {
        switch (curChar) {
          case '\\': break;
          case '&': strncpy(&m_str[start], "&amp;", 5); start += 5; break;
          case '<': strncpy(&m_str[start], "&lt;", 4); start += 4; break;
          case '>': strncpy(&m_str[start], "&gt;", 4); start += 4; break;
          case '\"': 
            if (oldChar == '\\') {
                strncpy(&m_str[start], "&quot;", 6); start += 6;
            }
            break;
          case '\'':
            if (oldChar == '\\') {
                strncpy(&m_str[start], "&apos;", 6); start += 6;
            }
            break;            
          default: m_str[start++] = curChar; break;
        }
        oldChar = curChar;
    }
    if (m_hasSpace) m_str[start++] = '\"';
}

myString::~myString()
{
    if (m_str == 0) return;
    delete m_str;
    m_str = 0;
    m_size = 0;
    m_capacity = 0;
}
    
void myString::init(const char * str, bool literal)
{
    if (str == 0) return;
    int size = m_size = strlen(str);
    if (literal) m_size = length(str, literal);
    if (m_block == 0) m_block = m_size+1;
    while (m_capacity < m_size+1) m_capacity += m_block;
    m_str = new char[m_capacity];
    
    if (literal) copy(str, literal, 0);
    else strncpy(m_str, str, m_size);

    m_str[m_size] = '\0';
    m_block += m_block;        
}

void myString::append(const char * str, bool literal)
{
    if (str == 0) return;
    int oldSize = m_size;
    int addSize = strlen(str);
    if (literal) addSize = length(str, literal);
    m_size += 1 + addSize;                          // reserve a space
    if (m_capacity < m_size+1) {
        char * oldStr = m_str;
        while (m_capacity < m_size+1) m_capacity += m_block;
        m_str = new char[m_capacity];
        strncpy(m_str, oldStr, oldSize);
        m_str[oldSize] = '\0';
        m_block += m_block;
        delete oldStr;
    }
    m_str[oldSize] = ' ';                           // concatenate space 
    if (literal) copy(str, literal, oldSize+1);
    else strncpy(&m_str[oldSize+1], str, addSize);  // concatenate new string
    m_str[m_size] = '\0';
}

